import {Component, OnInit} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {SpeciesAnalysis} from 'src/app/models/SpeciesAnalysis';
import {DataService} from 'src/app/services/data.service';
import {MultipleFamilyAnalysis} from "../../models/MultipleFamilyAnalysis";
import {ModelAnalysis} from "../../models/ModelAnalysis";
import {RegionModelAnalysis} from "../../models/RegionModelAnalysis";
import {RegionAnalysis} from "../../models/RegionAnalysis";
import {RegionTreeDistance} from "../../models/RegionTreeDistance";

@Component({
  selector: 'slf-result',
  templateUrl: './result.component.html',
  styleUrls: ['./result.component.scss']
})
export class ResultComponent implements OnInit {
  public id: string;
  public readonly family: string;
  private multipleFamilyAnalysis?: MultipleFamilyAnalysis;
  private speciesAnalysis?: SpeciesAnalysis;
  public speciesAnalysisNames: string[];

  public readonly displayedColumns: string[] =
    ['predicted', 'region', 'model', 'min_tree_distance', 'isoelectric_point'];

  constructor(
    private readonly route: ActivatedRoute,
    private readonly router: Router,
    private readonly dataService: DataService
  ) {
    const paramMap = this.route.snapshot.paramMap;
    if (!(paramMap.has('family') && paramMap.has('id'))) {
      this.router.navigate(['/']);
      throw new Error('Missing family and id');
    }
    this.family = <string>paramMap.get('family');
    this.id = <string>paramMap.get('id');
    this.speciesAnalysisNames = [];
  }

  public ngOnInit(): void {
    this.dataService.getMultipleFamilyAnalysis()
      .subscribe(mfa => {
        this.multipleFamilyAnalysis = mfa;
        this.speciesAnalysis = this.multipleFamilyAnalysis.all_results[this.family]
          .results[`slf/${this.family}/${this.id}`];
        this.speciesAnalysisNames.push(this.id);
      });
  }

  public listModelAnalysis(): ModelAnalysis[] {
    if (this.speciesAnalysis === undefined) {
      return []
    }

    return this.speciesAnalysis.region_analysis
      .map(ra => ra.model_analysis)
      .reduce((prev, curr) => prev.concat(curr));
  }

  public getModelRegion(analysis: ModelAnalysis): string {
    if (this.speciesAnalysis === undefined) {
      return '';
    }

    for (const regionAnalysis of this.speciesAnalysis.region_analysis) {
      if (regionAnalysis.model_analysis.includes(analysis)) {
        return regionAnalysis.region_name;
      }
    }

    return '';
  }

  public doesModelHavePredicted(analysis: ModelAnalysis): boolean {
    return analysis.predicted !== undefined;
  }

  public doesModelHavePutative(analysis: ModelAnalysis): boolean {
    return analysis.putative !== undefined;
  }

  public doesModelHaveBoth(analysis: ModelAnalysis): boolean {
    return this.doesModelHavePredicted(analysis) && this.doesModelHavePutative(analysis);
  }

  private getRegionModelAnalysis(analysis: ModelAnalysis): RegionModelAnalysis | undefined {
    let model: RegionModelAnalysis | undefined = undefined;

    if (analysis.predicted !== undefined) {
      model = analysis.predicted;
    } else if (analysis.putative !== undefined) {
      model = analysis.putative;
    }

    return model;
  }

  public getMinTreeDistance(analysis: ModelAnalysis): RegionTreeDistance[] {
    const model = this.getRegionModelAnalysis(analysis);

    if (model !== undefined && model.min_tree_distance !== undefined) {
      return model.min_tree_distance;
    } else {
      return [];
    }
  }

  public getIsoelectricPoints(analysis: ModelAnalysis): number[] {
    const model = this.getRegionModelAnalysis(analysis);

    if (model !== undefined && model.isoelectric_points !== undefined) {
      return model.isoelectric_points;
    } else {
      return [];
    }
  }

  public onChangeResults(value: string): void {
    //this.id = value
    //this.speciesAnalysis = this.multipleFamilyAnalysis.get(this.family!)?.get(value)
  }
}
